package org.apidesign.apifest08.currency;

import java.util.*;

final class MultiCurrencyConstantRateConverter implements CurrencyConverter {

	private final Map<Currency, Double> rates;

	public MultiCurrencyConstantRateConverter(Map<Currency, Double> rates) {
		this.rates = rates;
	}

	/**
	 * This is convenience method for convert(Currency.getInstance(from), Currency.getInstance(to)).
	 *
	 * @param value that should be converted form one currency to the other
	 * @param from  ISO-4217 code of the currency of the value provided
	 * @param to	ISO-4212 code of the currency to which the value should be converted
	 *
	 * @return value expressed in the target value
	 *
	 * @throws IllegalArgumentException if any of the arguments is not a valid ISO code
	 * @throws CurrencyConversionException
	 *                                  if the conversion cannot be performed with desired parameters, for
	 *                                  example the exchange rates are not current, connection to exchange rates
	 *                                  provider is not available
	 * @throws NullPointerException	 if any of the specified currency ISO codes is null
	 */
	// this method is provided to ensure future compatibility for converters supporting more than 2 currencies
	//		- simpler methods with fewer arguments would make using such converters less intuitive
	@Override
	public double convert(double value, /*@NotNull*/ String from, /*@NotNull*/ String to)
			throws CurrencyConversionException, NullPointerException, IllegalArgumentException {
		return convert(value, Currency.getInstance(from), Currency.getInstance(to));
	}

	/**
	 * Converts the specified value from one currency (from) to target currency (to).
	 *
	 * @param value that should be converted form one currency to the other
	 * @param from  ISO-4217 code of the currency of the value provided
	 * @param to	ISO-4212 code of the currency to which the value should be converted
	 *
	 * @return value expressed in the target value
	 *
	 * @throws IllegalArgumentException if any of the arguments is not a valid ISO code
	 * @throws CurrencyConversionException
	 *                                  if the conversion cannot be performed with desired parameters, for
	 *                                  example the exchange rates are not current, connection to exchange rates
	 *                                  provider is not available
	 * @throws NullPointerException	 if any of the specified currency ISO codes is null
	 */
	@Override
	public double convert(double value, /*@NotNull*/ Currency from, /*@NotNull*/ Currency to)
			throws CurrencyConversionException, NullPointerException {

		// this is not necessary, but we let users know that nulls are not allowed here - should be handled by annotations
		if (from == null || to == null)
			throw new NullPointerException("One of the specified currencies in null");

		Double fromRate = rates.get(from);
		Double toRate = rates.get(to);

		if (fromRate == null)
			throw new CurrencyConversionException(from, to, String.format("Currency %1$s not supported", from));
		if (toRate == null)
			throw new CurrencyConversionException(from, to, String.format("Currency %1$s not supported", to));

		return (value / fromRate) * toRate;
	}
}
